/* 
   Copyright (C) 2010, 2011 German A. Arias <german@xelalug.org>

   This file is part of Gemas application

   Gemas is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 3 of the License, or (at your option) any later version.
 
   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
 
   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import "GemasDocument.h"
#import "GemasEditorView.h"
#import "GNUstepGUI/GSTheme.h"
#import <Foundation/NSUserDefaults.h>
#import <HighlighterKit/HighlighterKit.h>

@interface GemasDocument (Private)

- (void) setupNewDefaults;

@end

@implementation GemasDocument (Private)

- (void) setupNewDefaults
{
  NSUserDefaults * df = [NSUserDefaults standardUserDefaults];
  NSData * data;

  ASSIGN (defaultFont, [HKSyntaxHighlighter defaultFont]);

  data = [df dataForKey: @"EditorTextColor"];
  if (data != nil)
    {
      ASSIGN (textColor, [NSKeyedUnarchiver unarchiveObjectWithData: data]);
    }

  data = [df dataForKey: @"EditorBackgroundColor"];
  if (data != nil)
    {
      ASSIGN (backgroundColor, [NSKeyedUnarchiver unarchiveObjectWithData:
        data]);
    }

  data = [df dataForKey: @"EditorInsertionPointColor"];
  if (data != nil)
    {
      ASSIGN (insertionPointColor, [NSKeyedUnarchiver
        unarchiveObjectWithData: data]);
    }


  if (textColor != nil)
    {
      [textView setTextColor: textColor];
    }
  if (backgroundColor != nil)
    {
      [textView setBackgroundColor: backgroundColor];
      [textView setDrawsBackground: YES];
    }
  if (insertionPointColor != nil)
    {
      [textView setInsertionPointColor: insertionPointColor];
    }

  [textView setNeedsDisplay: YES];
}

@end

@implementation GemasDocument

- (void) dealloc
{
  TEST_RELEASE (string);
  TEST_RELEASE (defaultFont);
  TEST_RELEASE (textColor);
  TEST_RELEASE (backgroundColor);
  TEST_RELEASE (insertionPointColor);

  [[NSNotificationCenter defaultCenter] removeObserver: self];

  [super dealloc];
}

- (id)init
{
  if ((self = [super init]) != nil)
    {
      [[NSNotificationCenter defaultCenter]
        addObserver: self
           selector: @selector (defaultsChanged:)
               name: CodeEditorDefaultsDidChangeNotification
             object: nil];

      return self;
    }
  else
    {
      return nil;
    }
}

- (BOOL) readFromFile: (NSString *) fileName ofType: (NSString *) fileType
{
  NSString * aString = [NSString stringWithContentsOfFile: fileName];
  NSString *insert = nil;

  if (aString != nil)
    {
      NSInteger tab = [[NSUserDefaults standardUserDefaults]
                        integerForKey: @"Indentation"];

      switch (tab)
        {
        case 0:
          insert = [NSString stringWithString: @"        "];
          break;
        case 1:
          insert = [NSString stringWithString: @"    "];
          break;
        case 2:
          insert = [NSString stringWithString: @"  "];
          break;
        }

      aString = [aString stringByReplacingOccurrencesOfString: @"\t" withString: insert];
      ASSIGN (string, aString);

      return YES;
    }
  else
    {
      return NO;
    }
}

- (BOOL) writeToFile: (NSString *) fileName ofType: (NSString *) fileType
{
  BOOL result;
  [textView breakUndoCoalescing];
  result = [[textView string] writeToFile: fileName atomically: NO];

  return result;
}

- (void) awakeFromNib
{
  NSMutableDictionary *typingAttrs;
  
  // don't allow users to change the font with the font panel
  [textView setFont: defaultFont];
  
  /* turn off ligatures - they're useless for a code editor */
  typingAttrs = [[[textView typingAttributes] mutableCopy] autorelease];
  [typingAttrs setObject: [NSNumber numberWithInt: 0]
               forKey: NSLigatureAttributeName];
  [textView setTypingAttributes: typingAttrs];
  [textView setAllowsUndo: YES];
  [textView setUsesFindPanel: YES];
  
  [textView replaceCharactersInRange: NSMakeRange(0, 0) withString: string];
  [textView setSelectedRange: NSMakeRange(0, 0)];
  DESTROY(string);
  
  //Setup the user preferences
  [self setupNewDefaults];
  
  //Set the syntax highlighter
  [textView createSyntaxHighlighterForFileType: [self fileType]];
  
  //Handle the Windows menu, to prevent crash with menu in window
  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) ==
      NSWindows95InterfaceStyle)
    {
      id window;
      NSMenu *windowsMenu;
      NSEnumerator *windows = [[NSApp windows] objectEnumerator];
      
      while((window = [windows nextObject]))
	{
	  if ((window != documentWindow) && [window menu] != nil)
	    {
	      NSMenuItem *new = AUTORELEASE([NSMenuItem new]);
	      [new setTitle: [self displayName]];
	      [new setTarget: documentWindow];
	      [new setAction: @selector(makeKeyAndOrderFront:)];
	      [new setImage: [NSImage imageNamed: @"common_Close"]];
	      
	      windowsMenu = [[[window menu] itemWithTitle: _(@"Windows")] submenu];
	      [windowsMenu insertItem: new atIndex: 1];
	    }
	}
    }
}

- (NSString *) windowNibName
{
  return @"Editor";
}

- (void) goToLineNumber: (int)number
{
  NSUInteger offset = 0;
  NSUInteger i;
  NSUInteger lineNumber;
  NSRange rLine;
  NSString *line;
  NSArray *lines = [[textView string] componentsSeparatedByString: @"\n"];
  NSEnumerator *search = [lines objectEnumerator];


  if (number < 0)
    {
      lineNumber = -1*number;
    }
  else
    {
      lineNumber = number;
    }

  for (i=1; (line = [search nextObject]) != nil && i < lineNumber;
       i++)
    {
      offset += [line length] + 1;
    }
  
  if (line != nil)
    {
      rLine = NSMakeRange(offset, [line length]);
    }
  else
    {
      rLine = NSMakeRange([[textView string] length], 0);
    }

  [textView setSelectedRange: rLine];
  [textView scrollRangeToVisible: rLine];
}

- (void) textViewDidChangeSelection: (NSNotification *) notification
{
  //Number of line and column
  NSRange tex = [textView selectedRange];
  NSArray *selectedLines = [[[textView string] substringWithRange: NSMakeRange(0, tex.location)] componentsSeparatedByString: @"\n"];

  [numberLine setIntValue: [selectedLines count]];
  [numberColumn setIntValue: [[selectedLines lastObject] length] + 1];
}

- (BOOL) revertToContentsOfURL: (NSURL*)url ofType: (NSString*)type error: (NSError**)error
{
  NSString * aString = [NSString stringWithContentsOfURL: url];
  NSString *insert = nil;
  NSMutableDictionary *typingAttrs;
  
  if (aString != nil)
    {
      NSInteger tab = [[NSUserDefaults standardUserDefaults]
                        integerForKey: @"Indentation"];
      
      switch (tab)
        {
        case 0:
          insert = [NSString stringWithString: @"        "];
          break;
        case 1:
          insert = [NSString stringWithString: @"    "];
          break;
        case 2:
          insert = [NSString stringWithString: @"  "];
          break;
        }
      
      aString = [aString stringByReplacingOccurrencesOfString: @"\t" withString: insert];
      
    }
  else
    {
      return NO;
    }
  
  // don't allow users to change the font with the font panel
  [textView setFont: defaultFont];
  
  /* turn off ligatures - they're useless for a code editor */
  typingAttrs = [[[textView typingAttributes] mutableCopy] autorelease];
  [typingAttrs setObject: [NSNumber numberWithInt: 0]
               forKey: NSLigatureAttributeName];
  [textView setTypingAttributes: typingAttrs];
  [textView setAllowsUndo: YES];
  [textView setUsesFindPanel: YES];
  
  [textView replaceCharactersInRange: NSMakeRange(0, [[textView string] length])
	    withString: aString];
  [textView setSelectedRange: NSMakeRange(0, 0)];
  
  //Setup the user preferences
  [self setupNewDefaults];
  
  //Set the syntax highlighter
  [textView createSyntaxHighlighterForFileType: [self fileType]];
  
  return YES;
}

- (NSColor *) textColor
{
  return textColor;
}

- (void) defaultsChanged: (NSNotification *) notif
{
  [self setupNewDefaults];
}

- (void) windowDidBecomeMain: (NSNotification*)aNotification
{
  //Handle the status of documents in Windows menu
  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) ==
      NSWindows95InterfaceStyle)
    {
      int x, y;
      id document, window;
      NSMenu *windowsMenu, *recentDocsMenu;
      NSMenuItem *item, *clear;
      NSEnumerator *documents = [[[NSDocumentController sharedDocumentController]
				   documents] objectEnumerator];
      /* Get list of recent documents */
      NSArray *recentDocuments = [[NSUserDefaults standardUserDefaults] 
				   objectForKey: @"NSRecentDocuments"];

      windowsMenu = [[[documentWindow menu] itemWithTitle: _(@"Windows")] submenu];
      
      while((document = [documents nextObject]))
	{
	  if (document != self)
	    {
	      item = [windowsMenu itemWithTitle: [document displayName]];

	      if ([document isDocumentEdited])
		{
		  [item setImage: [NSImage imageNamed: @"common_CloseBroken"]];
		}
	      else
		{
		  [item setImage: [NSImage imageNamed: @"common_Close"]];
		}
	    }
	}

      NSEnumerator *windows = [[NSApp windows] objectEnumerator];
      
      while((window = [windows nextObject]))
	{
	  recentDocsMenu = [[[[[window menu] itemWithTitle: _(@"Document")] submenu]
			      itemWithTitle: _(@"Open Recent")] submenu];
	  clear = [recentDocsMenu itemWithTitle: _(@"Clear List")];
	  
	  if ([[recentDocsMenu itemArray] count] > 2)
	    {
	      y = [[recentDocsMenu itemArray] count] - 3;
	      for (x = 0; x <= y; x++)
		{
		  [recentDocsMenu removeItemAtIndex: 0];
		}
	    }
	  else
	    {
	      if ([recentDocuments count] > 0)
		{
		  [recentDocsMenu insertItem: [NSMenuItem separatorItem] atIndex: 0];
		  [clear setEnabled: YES];
		}
	    }
	  
	  [clear setTarget: self];
	  [clear setAction: @selector(clear:)];
	  
	  if ([recentDocuments count] > 0)
	    {
	      for (x = 0; x <= [recentDocuments count] - 1; x++)
		{
		  NSMenuItem *newDoc = AUTORELEASE([NSMenuItem new]);
		  [newDoc setTitle: [[recentDocuments objectAtIndex: x] lastPathComponent]];
		  [newDoc setTag: x];
		  [newDoc setTarget: [NSDocumentController sharedDocumentController]];
		  [newDoc setAction: @selector(_openRecentDocument:)];
		  [recentDocsMenu insertItem: newDoc atIndex: 0];
		}
	    }
	}
    }
}

- (void) windowWillClose: (NSNotification*)aNotification
{
  //Handle the Windows menu, to prevent crash with menu in window
  if (NSInterfaceStyleForKey(@"NSMenuInterfaceStyle", nil) ==
      NSWindows95InterfaceStyle)
    {
      int x;
      id window;
      NSMenu *windowsMenu;
      NSEnumerator *windows = [[NSApp windows] objectEnumerator];
      
      while((window = [windows nextObject]))
	{
	  if ((window != documentWindow) && ([window menu] != nil))
	    {
	      windowsMenu = [[[window menu] itemWithTitle: _(@"Windows")] submenu];
	      x = [windowsMenu indexOfItemWithTitle: [self displayName]];
	      [windowsMenu removeItemAtIndex: x];
	    }
	}
    }
}

- (void) clear: (id)sender
{
  int x, y;
  NSMenu *recentDocsMenu;
  NSWindow *window;
  
  [[NSUserDefaults standardUserDefaults] 
    setObject: [NSMutableArray array] forKey: @"NSRecentDocuments"];
  [[NSDocumentController sharedDocumentController] clearRecentDocuments: self];
  
  NSEnumerator *windows = [[NSApp windows] objectEnumerator];
  
  while ((window = [windows nextObject]))
    {
      
      recentDocsMenu = [[[[[window menu] itemWithTitle: _(@"Document")] submenu]
			  itemWithTitle: _(@"Open Recent")] submenu];
      [[recentDocsMenu itemWithTitle: _(@"Clear List")] setEnabled: NO];
      
      if ([[recentDocsMenu itemArray] count] > 2)
	{
	  y = [[recentDocsMenu itemArray] count] - 2;
	  for (x = 0; x <= y; x++)
	    {
	      [recentDocsMenu removeItemAtIndex: 0];
	    }
	}
    }
}

@end
